/* {{{ Copyright etc. */

/*  tsplot - FMRI time series and model plotting

    Stephen Smith, FMRIB Image Analysis Group

    Copyright (C) 1999-2000 University of Oxford  */

/*  Part of FSL - FMRIB's Software Library
    WWW:      http://www.fmrib.ox.ac.uk/fsl
    Email:    fsl@fmrib.ox.ac.uk
    
    Developed at FMRIB (Oxford Centre for Functional Magnetic Resonance
    Imaging of the Brain), Department of Clinical Neurology, Oxford
    University, Oxford, UK
    
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or (at
    your option) any later version.
    
    This program is distributed in the hope that it will be useful, but in
    order that the University as a charitable foundation protects its
    assets for the benefit of its educational and research purposes, the
    University makes clear that no condition is made or to be implied, nor
    is any warranty given or to be implied, as to the accuracy of FSL, or
    that it will be suitable for any particular purpose or for use under
    any specific conditions.  Furthermore, the University disclaims all
    responsibility for the use which is made of FSL.  See the GNU General
    Public License for more details.
    
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
    USA */

/* }}} */
/* {{{ defines, includes and typedefs */

#include "libss.h"
#include "libavw.h"

/* }}} */
/* {{{ usage */

void usage(void)
{
  printf("Usage: tsplot <feat_directory.feat> [options]\n");
  printf("[-f <4D_data>] input main filtered data, in case it's not in the FEAT directory\n");
  printf("[-p <+_string>] string of pluses to insert\n");
  printf("[-c <X Y Z>] : use X,Y,Z instead of max Z stat position\n");
  printf("[-C <X Y Z output_file.txt>] : use X,Y,Z to output time series only - no stats or modelling\n");
  printf("[-o <output_directory>] change output directory from default of input feat directory\n");
  exit(1);
}

/* }}} */
/* {{{ find_key */

char *find_key(FILE *fd, char *control_line, const char *key)
{
  while (fgets(control_line, 1000, fd)!=NULL)
    if ( (strncmp(control_line,key,strlen(key))==0) )
      {
	return control_line+strlen(key);
      }
	  
  printf("Error: key \"%s\" not found.\n",key);
  exit(1);
}

/* }}} */
/* {{{ read_model */

double *read_model(char *filename, int *nevs, int *npts)
{
  FILE *designfp;
  char control_line[1010];
  double *model;
  int i;

  if((designfp=fopen(filename,"r"))==NULL)
    {
      fprintf(stderr,"Can't open %s\n",filename);
      exit(1);
    }

  *nevs=atof(find_key(designfp,control_line,"/NumWaves"));
  *npts=atof(find_key(designfp,control_line,"/NumPoints"));

  atof(find_key(designfp,control_line,"/Matrix"));

  model=(double*)malloc(sizeof(double)* *nevs * *npts);

  for(i=0; i<*nevs * *npts; i++)
    fscanf(designfp,"%lf",&model[i]);

  return model;
}

/* }}} */
/* {{{ read_contrasts */

double *read_contrasts(char *filename, int *nevs, int *ncon)
{
  FILE *designfp;
  char control_line[1010];
  double *contrasts;
  int i;

  if((designfp=fopen(filename,"r"))==NULL)
    {
      fprintf(stderr,"Can't open %s\n",filename);
      exit(1);
    }

  *nevs=atof(find_key(designfp,control_line,"/NumWaves"));
  *ncon=atof(find_key(designfp,control_line,"/NumContrasts"));

  atof(find_key(designfp,control_line,"/Matrix"));

  contrasts=(double*)malloc(sizeof(double)* *nevs * *ncon);

  for(i=0; i<*nevs * *ncon; i++)
    fscanf(designfp,"%lf",&contrasts[i]);

  return contrasts;
}

/* }}} */
/* {{{ main */

int main(int argc, char **argv)
{
  /* {{{ variables */

FDT          *im, *impe[1000], *imcope, *imz, *imclus;
image_struct image;
FILE         *ofp;
int          argi=1, ev, i, x, y, z, t, X=0, Y=0, Z=0, nevs, npts, ncon, size, ymin, ymax, coordset=0, dataonly=0;
double       *model, *contrasts, tsmean, tmpf, maxz;
char         fmridata[10000], featdir[10000], outputdir[10000], plusstring[10000], gpstart[10000],
             thestring[10000], thestring2[10000], thestring3[10000], datafile[10000], binstr[10000];

/* }}} */

  /* {{{ process arguments */

if (argc<2) usage();

strcpy(featdir,argv[argi++]);

strcpy(outputdir,featdir);

sprintf(fmridata,"%s/filtered_func_data",featdir);

/* will use binstr to work out where "gnuplot" and "convert" are, for later system calls */
sprintf(binstr,"%s",argv[0]);
binstr[strlen(binstr)-strlen("tsplot")]=0;

for (;argi<argc;argi++)
{
  if (!strcmp(argv[argi], "-f"))
    /* {{{ alternative fmri data */

{
  argi++;
  if (argc<argi+1)
    {
      printf("Error: no value given following -f\n");
      usage();
    }
  strcpy(fmridata,argv[argi]);
}

/* }}} */
  if (!strcmp(argv[argi], "-p"))
    /* {{{ plus string */

{
  argi++;
  if (argc<argi+1)
    {
      printf("Error: no value given following -p\n");
      usage();
    }
  strcpy(plusstring,argv[argi]);
}

/* }}} */
  else if (!strcmp(argv[argi], "-c"))
    /* {{{ alternative voxel position */

{
  coordset=1;
  argi++;
  if (argc<argi+3) /* options following c haven't been given */
    {
      printf("Error: incomplete values given following -c\n");
      usage();
    }
  X=atoi(argv[argi++]);
  Y=atoi(argv[argi++]);
  Z=atoi(argv[argi]);
}

/* }}} */
  else if (!strcmp(argv[argi], "-C"))
    /* {{{ output data only */

{
  coordset=1;
  dataonly=1;
  argi++;
  if (argc<argi+4)
    {
      printf("Error: incomplete values given following -C\n");
      usage();
    }
  X=atoi(argv[argi++]);
  Y=atoi(argv[argi++]);
  Z=atoi(argv[argi++]);
  strcpy(datafile,argv[argi]);
}

/* }}} */
  if (!strcmp(argv[argi], "-o"))
    /* {{{ output dir */

{
  argi++;
  if (argc<argi+1)
    {
      printf("Error: no value given following -o\n");
      usage();
    }
  strcpy(outputdir,argv[argi]);
}

/* }}} */
}

/* }}} */
  /* {{{ read filtered_func_data */

avw_read(fmridata,&image);
im=image.i;
/*printf("FMRI dimensions %d %d %d %d\n",image.x,image.y,image.t);*/

size=image.x*image.y*image.z;

if (dataonly)
  /* {{{ output raw data and exit */

{
  if((ofp=fopen(datafile,"w"))==NULL)
    {
      fprintf(stderr,"Can't open output data file %s\n",datafile);
      exit(1);
    }

  for(t=0; t<image.t; t++)
    fprintf(ofp,"%e\n",(float)im[t*size+Z*image.x*image.y+Y*image.x+X]);

  fclose(ofp);

  return 0;
}

/* }}} */

/* }}} */
  /* {{{ read design.mat */

sprintf(thestring,"%s/design%s.mat",featdir,plusstring);
model=read_model(thestring,&nevs,&npts);
/*printf("%d %d\n",nevs,npts);*/

npts=image.t;

/* }}} */
  /* {{{ read design.con */

sprintf(thestring,"%s/design%s.con",featdir,plusstring);
contrasts=read_contrasts(thestring,&nevs,&ncon);

/*printf("%d %d\n",nevs,ncon);
for(i=0;i<ncon;i++)
{
  for(ev=0;ev<nevs;ev++)
    printf("%e ",contrasts[i*nevs+ev]);
  printf("\n");
}*/

/* }}} */
  /* {{{ read PEs */

for(i=1;i<=nevs;i++)
{
  sprintf(thestring,"%s/stats/pe%d",featdir,i);
  /*printf("Reading PE image %s\n",thestring);*/
  avw_read(thestring,&image);
  impe[i-1]=image.i;
}

/* }}} */
  /* {{{ create plot(s) for each contrast */

for(i=0; i<ncon; i++)
{
  double *data=(double*)calloc(npts,sizeof(double));

  /* {{{ read COPE and derived stats */

{
  FILE *ifp;

  sprintf(thestring,"%s/stats/cope%s%d",featdir,plusstring,i+1);
  /*printf("Reading COPE image %s\n",thestring);*/
  avw_read(thestring,&image);
  imcope=image.i;

  sprintf(thestring,"%s/stats/zstat%s%d",featdir,plusstring,i+1);
  /*printf("Reading Z stat image %s\n",thestring);*/
  avw_read(thestring,&image);
  imz=image.i;

  imclus=NULL;
  if (!coordset)
    {
      sprintf(thestring,"%s/cluster_mask_zstat%s%d.hdr",featdir,plusstring,i+1);
      if((ifp=fopen(thestring,"r"))!=NULL)
	{
	  fclose(ifp);
	  sprintf(thestring,"%s/cluster_mask_zstat%s%d",featdir,plusstring,i+1);
	  /*printf("Reading cluster image %s\n",thestring);*/
	  avw_read(thestring,&image);
	  imclus=image.i;
	}
    }
}

/* }}} */

  /* {{{ open data file */

sprintf(thestring,"%s/tsplot%s%d.txt",outputdir,plusstring,i+1);

if((ofp=fopen(thestring,"w"))==NULL)
{
  fprintf(stderr,"Can't open output data file %s\n",thestring);
  exit(1);
}

/* }}} */
  /* {{{ find max Z and X,Y,Z */

if (!coordset)
{
  X=Y=Z=0;
  maxz=imz[0];

  for(z=0; z<image.z; z++)
    for(y=0; y<image.y; y++)
      for(x=0; x<image.x; x++)
	if ( (imz[z*image.x*image.y+y*image.x+x]>maxz) &&
	     ( (imclus==NULL) ||   /* make max Z be inside a cluster if we found a cluster map */
	       (imclus[z*image.x*image.y+y*image.x+x]>0) ) )
	  {
	    maxz=imz[z*image.x*image.y+y*image.x+x];
	    X=x; Y=y; Z=z;
	  }

  /*printf("Z stat %d has maximum of %f at %d %d %d\n",i+1,maxz,X,Y,Z);*/
}
else
  maxz=imz[Z*image.x*image.y+Y*image.x+X];

/* }}} */
  /* {{{ create 1D data vector and find mean */

tsmean=0;

for(t=0; t<npts; t++)
{
  data[t]=im[t*size+Z*image.x*image.y+Y*image.x+X];
  tsmean+=data[t];
}

tsmean/=npts;

/* }}} */
  /* {{{ output the data and model */

ymin=ymax=data[0];

for(t=0; t<npts; t++)
{
  double fullmodel=0, copemodel=0;

  for(ev=0; ev<nevs; ev++)
    {
      tmpf=model[t*nevs+ev]*impe[ev][Z*image.x*image.y+Y*image.x+X];
      fullmodel += tmpf;
      copemodel += tmpf*contrasts[i*nevs+ev];
    }
      
  fprintf(ofp,"%e ",data[t]);
  ymin=MIN(data[t],ymin);
  ymax=MAX(data[t],ymax);

  fprintf(ofp,"%e ",tsmean+copemodel);
  ymin=MIN(tsmean+copemodel,ymin);
  ymax=MAX(tsmean+copemodel,ymax);

  fprintf(ofp,"%e ",tsmean+fullmodel);
  ymin=MIN(tsmean+fullmodel,ymin);
  ymax=MAX(tsmean+fullmodel,ymax);

  fprintf(ofp,"%e ",data[t]-fullmodel+copemodel);

  fprintf(ofp,"\n");
}

ymax+=(ymax-ymin)/5;
ymin-=(ymax-ymin)/20;

/* }}} */
  /* {{{ call gnuplot via system call */

fclose(ofp);

thestring2[0]=0;
for(ev=0;ev<nevs;ev++)
{
  if ((float)((int)contrasts[i*nevs+ev])==contrasts[i*nevs+ev])
    sprintf(thestring2,"%s %d",thestring2,(int)contrasts[i*nevs+ev]);
  else
    sprintf(thestring2,"%s %.1f",thestring2,contrasts[i*nevs+ev]);
}

sprintf(gpstart,"echo \"set term pbm color\nset size %f,0.5\nset title 'Contrast %d [%s ] :",MAX(npts*4,600)/640.0,i+1,thestring2);

sprintf(thestring2,"%s/tsplot%s%d",outputdir,plusstring,i+1);

if (!coordset)
     sprintf(thestring3,"%s max Z stat of %.3f at voxel",gpstart,maxz);
else
     sprintf(thestring3,"%s Z stat of %.3f at selected voxel",gpstart,maxz);

sprintf(thestring,"%s (%d,%d,%d)'\nset yrange [%d:%d]\nplot '%s.txt' using 1 title 'data' with lines , '%s.txt' using 2 title 'cope partial model fit' with lines , '%s.txt' using 3 title 'full model fit' with lines \" | %sgnuplot | %sconvert - %s.gif",thestring3,X,Y,Z,ymin,ymax,thestring2,thestring2,thestring2,binstr,binstr,thestring2);

system(thestring);

sprintf(thestring,"%s (%d,%d,%d)'\nset yrange [%d:%d]\nplot '%s.txt' using 4 title 'reduced data' with lines , '%s.txt' using 2 title 'cope partial model fit' with lines \" | %sgnuplot | %sconvert - %sp.gif",thestring3,X,Y,Z,ymin,ymax,thestring2,thestring2,binstr,binstr,thestring2);

system(thestring);

/* }}} */

  if (imclus!=NULL)
    {
      double *fullmodel=(double*)calloc(npts,sizeof(double)),
	*copemodel=(double*)calloc(npts,sizeof(double)),
	*data=(double*)calloc(npts,sizeof(double));
      int count=0;

      /* {{{ open data file */

sprintf(thestring,"%s/tsplotc%s%d.txt",outputdir,plusstring,i+1);

if((ofp=fopen(thestring,"w"))==NULL)
{
  fprintf(stderr,"Can't open output data file %s\n",thestring);
  exit(1);
}

/* }}} */
      /* {{{ create 1D data vector and find mean */

for(x=0;x<size;x++)
  if (imclus[x]>0)
  {
    count++;
    for(t=0; t<npts; t++)
      data[t]+=im[t*size+x];
  }
/*printf("count=%d\n",count);*/

tsmean=0;
for(t=0; t<npts; t++)
{
  data[t]/=count;
  tsmean+=data[t];
}
tsmean/=npts;

ymin=ymax=data[0];

for(x=0;x<size;x++)
  if (imclus[x]>0)
    for(t=0; t<npts; t++)
	for(ev=0; ev<nevs; ev++)
	  {
	    tmpf=model[t*nevs+ev]*impe[ev][x];
	    fullmodel[t] += tmpf;
	    copemodel[t] += tmpf*contrasts[i*nevs+ev];
	  }

for(t=0; t<npts; t++)
{
  fullmodel[t] /= count;
  copemodel[t] /= count;

  fprintf(ofp,"%e ",data[t]);
  ymin=MIN(data[t],ymin);
  ymax=MAX(data[t],ymax);

  fprintf(ofp,"%e ",tsmean+copemodel[t]);
  ymin=MIN(tsmean+copemodel[t],ymin);
  ymax=MAX(tsmean+copemodel[t],ymax);

  fprintf(ofp,"%e ",tsmean+fullmodel[t]);
  ymin=MIN(tsmean+fullmodel[t],ymin);
  ymax=MAX(tsmean+fullmodel[t],ymax);
	
  fprintf(ofp,"%e ",data[t]-fullmodel[t]+copemodel[t]);

  fprintf(ofp,"\n");
}

ymax+=(ymax-ymin)/5;
ymin-=(ymax-ymin)/20;

/* }}} */
      /* {{{ call gnuplot via system call */

fclose(ofp);

sprintf(thestring2,"%s/tsplotc%s%d",outputdir,plusstring,i+1);

sprintf(thestring,"%s averaged over %d voxels'\nset yrange [%d:%d]\nplot '%s.txt' using 1 title 'data' with lines , '%s.txt' using 2 title 'cope partial model fit' with lines , '%s.txt' using 3 title 'full model fit' with lines \" | %sgnuplot | %sconvert - %s.gif",gpstart,count,ymin,ymax,thestring2,thestring2,thestring2,binstr,binstr,thestring2);

system(thestring);

sprintf(thestring,"%s averaged over %d voxels'\nset yrange [%d:%d]\nplot '%s.txt' using 4 title 'reduced data' with lines , '%s.txt' using 2 title 'cope partial model fit' with lines \" | %sgnuplot | %sconvert - %sp.gif",gpstart,count,ymin,ymax,thestring2,thestring2,binstr,binstr,thestring2);

system(thestring);

/* }}} */
    }
}

/* }}} */

  return 0;
}

/* }}} */
